from datetime import datetime

import mock
from bson import ObjectId
from cryptomodel.coinmarket import prices
from cryptomodel.cryptomodel import user_transaction, exchange_rates

from cryptodataaccess.Rates.RatesMongoStore import RatesMongoStore
from cryptodataaccess.Rates.RatesRepository import RatesRepository
from cryptodataaccess.config import configure_app
import pytest
from cryptodataaccess.helpers import do_local_connect, convert_to_int_timestamp
from cryptodataaccess.tests.helpers import insert_prices_record, insert_exchange_record, insert_prices_record_with_method, \
    get_prices20200812039_record, get_prices20200801T2139_record



DATE_FORMAT = '%Y-%m-%d %H:%M:%S.%f'


@pytest.fixture(scope='module')
def mock_log():
    with mock.patch("cryptodataaccess.helpers.log_error"
                    ) as _mock:
        _mock.return_value = True
        yield _mock


def test_fetch_symbol_rates_for_date_pass_str_or_dt():
    config = configure_app()
    do_local_connect(config)
    user_transaction.objects.all().delete()
    exchange_rates.objects.all().delete()
    prices.objects.all().delete()

    insert_prices_record()
    insert_exchange_record()

    config = configure_app()
    store = RatesMongoStore(config, mock_log)
    rates_repo = RatesRepository(store)
    do_local_connect(config)
    rates_repo.fetch_symbol_rates_for_date(convert_to_int_timestamp(datetime.today()))
    dt = datetime(year=2040, month=1, day=1)
    rates_repo.fetch_symbol_rates_for_date(convert_to_int_timestamp(dt))
    assert (len(prices.objects) == 1)


def test_fetch_symbol_rates_for_dat_with_two_entries_within_two_hours():
    config = configure_app()
    users_store = RatesMongoStore(config, mock_log)
    rates_repo = RatesRepository(users_store)
    do_local_connect(config)
    dt_now = convert_to_int_timestamp(datetime.today())
    user_transaction.objects.all().delete()
    prices.objects.all().delete()

    insert_exchange_record()
    insert_prices_record_with_method(get_prices20200812039_record)
    insert_prices_record_with_method(get_prices20200801T2139_record)
    rts = rates_repo.fetch_symbol_rates_for_date(convert_to_int_timestamp(datetime.today()))
    dt = datetime(year=2020, month=8, day=1, hour=21, minute=0) #1596304800
    print(dt.timestamp())


def test_delete_symbol_rates():
    config = configure_app()
    users_store = RatesMongoStore(config, mock_log)
    rates_repo = RatesRepository(users_store)
    do_local_connect(config)
    prices.objects.all().delete()
    new_price = prices()
    new_price.source_id = ObjectId('666f6f2d6261722d71757578')
    rates_repo.insert_prices(new_price.source_id, None , None , new_price.source_id)
    dt = convert_to_int_timestamp(datetime(year=2025, month=7, day=3))
    theprices =  prices.objects.all()
    assert (len(    prices.objects) == 1 )
    rates_repo.delete_prices(new_price.source_id)
    theprices2 =  rates_repo.fetch_latest_prices_to_date(convert_to_int_timestamp(datetime.today()))
    assert (len(    prices.objects) == 0 )











from datetime import datetime

import mock
from bson import ObjectId
from cryptomodel.computed_notification import computed_notification
from cryptomodel.operations import OPERATIONS
from cryptomodel.cryptostore import user_channel, user_notification, user_settings
from cryptomodel.coinmarket import prices
from cryptomodel.fixer import exchange_rates

from cryptodataaccess.Calculator.CalculatorMongoStore import CalculatorMongoStore
from cryptodataaccess.Calculator.CalculatorRepository import CalculatorRepository
from cryptodataaccess.Memory import USER_NOTIFICATIONS_MEMORY_KEY, USER_SETTINGS_MEMORY_KEY, USER_CHANNELS_MEMORY_KEY, \
    CALCULATOR_MEMORY_KEY
from cryptodataaccess.Rates.RatesMongoStore import RatesMongoStore
from cryptodataaccess.Users.UsersMongoStore import UsersMongoStore
from cryptodataaccess.config import configure_app
from cryptodataaccess.Users.UsersRepository import UsersRepository
from cryptodataaccess.Rates.RatesRepository import RatesRepository
import pytest

from cryptodataaccess.helpers import do_local_connect


@pytest.fixture(scope='module')
def mock_log():
    with mock.patch("cryptodataaccess.helpers.log_error"
                    ) as _mock:
        _mock.return_value = True
        yield _mock


def test_insert_computed_notification():
    config = configure_app()
    store = CalculatorMongoStore(config, mock_log)
    repo = CalculatorRepository(store)
    do_local_connect(config)
    computed_notification.objects.all().delete()
    ut = repo.add_computed_notification(user_id=1,user_name="ds",user_email="ds",
                                        notification_type="BALANCE",check_every="00:01",
                                        is_active=True,
                                        computed_date="2020-01-01",
                                        end_date="2020-01-01",
                                        start_date="2020-01-01",
                                        threshold_value=1,channel_type="TELEGRAM",
                                        result="sss",
                                        source_id=ObjectId('666f6f2d6261722d71757578')

                                        )
    repo.commit()

    cn = repo.memories[CALCULATOR_MEMORY_KEY].items[0]
    assert (cn.user_id == 1)
    assert (len(computed_notification.objects) == 1)
    assert (cn.operation == OPERATIONS.ADDED.name)


    ut = repo.add_computed_notification(user_id=1,user_name="ds",user_email="ds",
                                        notification_type="BALANCE",check_every="00:01",
                                        is_active=True,
                                        computed_date="2020-01-01",
                                        end_date="2020-01-01",
                                        start_date="2020-01-01",
                                        threshold_value=1,channel_type="TELEGRAM",
                                        result="sss",
                                        source_id=ObjectId('666f6f2d6261722d71757578')

                                        )
    repo.commit()
    cn = repo.memories[CALCULATOR_MEMORY_KEY].items[0]
    assert (cn.user_id == 1)
    assert (len(computed_notification.objects) == 2)
    assert (cn.operation == OPERATIONS.ADDED.name)



from abc import ABCMeta, abstractmethod


class NotificationsStore(metaclass=ABCMeta):

    @abstractmethod
    def fetch_unsent_notifications(self, user_id, user_name, user_email, notification_type, check_every, is_active,
                              start_date, end_date, channel_type, threshold_value, source_id, computed_date, result,
                              sent=False):
        pass

    @abstractmethod
    def insert_sent_notification(self):
        pass






import os
from keyring import get_password
from keyrings.alt.file import PlaintextKeyring
from werkzeug.utils import import_string
import keyring.backend

DB = "calculator_service"
PORT = 27017
LOCAL_SERVER = "127.0.0.1"
KAFKA_BROKERS = "134.122.79.43:9092"
CENTRAL_SERVER = "134.122.79.43"
CENTRAL_MONGO_DATABASE = "admin"
TRANSACTIONS_TOPIC_NAME = "transactions"
COMPUTED_NOTIFICATIONS_TOPIC_NAME = "computed_notifications"
USER_NOTIFICATIONS_TOPIC_NAME = "user_notifications"
LOCAL_MONGO_DATABASE = "admin"


class BaseConfig(object):
    DEBUG = False
    TESTING = False
    CENTRAL_SERVER = CENTRAL_SERVER
    CENTRAL_MONGO_DATABASE = CENTRAL_MONGO_DATABASE
    CENTRAL_MONGO_USERNAME = ""
    CENTRAL_MONGO_PASSWORD = ""
    CENTRAL_PORT = PORT
    LOCAL_PORT = PORT
    LOCAL_SERVER = LOCAL_SERVER
    LOCAL_MONGO_DATABASE = LOCAL_MONGO_DATABASE
    LOCAL_MONGO_USERNAME = ""
    LOCAL_MONGO_PASSWORD = ""
    LOGS_PATH = 'cryptodataaccess/logs/cryptodataaccess.log'
    KAFKA_BROKERS = KAFKA_BROKERS
    TRANSACTIONS_TOPIC_NAME = TRANSACTIONS_TOPIC_NAME
    USER_NOTIFICATIONS_TOPIC_NAME = USER_NOTIFICATIONS_TOPIC_NAME
    COMPUTED_NOTIFICATIONS_TOPIC_NAME = COMPUTED_NOTIFICATIONS_TOPIC_NAME


class DevelopmentConfig(BaseConfig):
    DEBUG = False
    TESTING = False
    CENTRAL_SERVER = CENTRAL_SERVER
    CENTRAL_MONGO_DATABASE = CENTRAL_MONGO_DATABASE
    CENTRAL_MONGO_USERNAME = ""
    CENTRAL_MONGO_PASSWORD = ""
    CENTRAL_PORT = PORT
    LOCAL_PORT = PORT
    LOCAL_SERVER = LOCAL_SERVER
    LOCAL_MONGO_DATABASE = LOCAL_MONGO_DATABASE
    LOCAL_MONGO_USERNAME = ""
    LOCAL_MONGO_PASSWORD = ""
    LOGS_PATH = 'cryptodataaccess/logs/cryptodataaccess.log'
    KAFKA_BROKERS = KAFKA_BROKERS
    TRANSACTIONS_TOPIC_NAME = TRANSACTIONS_TOPIC_NAME
    USER_NOTIFICATIONS_TOPIC_NAME = USER_NOTIFICATIONS_TOPIC_NAME
    COMPUTED_NOTIFICATIONS_TOPIC_NAME = COMPUTED_NOTIFICATIONS_TOPIC_NAME


class ProductionConfig(BaseConfig):
    DEBUG = False
    TESTING = False
    CENTRAL_SERVER = CENTRAL_SERVER
    CENTRAL_MONGO_DATABASE = CENTRAL_MONGO_DATABASE
    CENTRAL_MONGO_USERNAME = ""
    CENTRAL_MONGO_PASSWORD = ""
    CENTRAL_PORT = PORT
    LOCAL_PORT = PORT
    LOCAL_SERVER = LOCAL_SERVER
    LOCAL_MONGO_DATABASE = LOCAL_MONGO_DATABASE
    LOCAL_MONGO_USERNAME = ""
    LOCAL_MONGO_PASSWORD = ""
    LOGS_PATH = 'cryptodataaccess/logs/cryptodataaccess.log'
    KAFKA_BROKERS = KAFKA_BROKERS
    TRANSACTIONS_TOPIC_NAME = TRANSACTIONS_TOPIC_NAME
    USER_NOTIFICATIONS_TOPIC_NAME = USER_NOTIFICATIONS_TOPIC_NAME
    COMPUTED_NOTIFICATIONS_TOPIC_NAME = COMPUTED_NOTIFICATIONS_TOPIC_NAME


config = {
    "development": "cryptodataaccess.config.DevelopmentConfig",
    "production": "cryptodataaccess.config.ProductionConfig",
    "default": "cryptodataaccess.config.DevelopmentConfig",
}


def configure_app():
    keyring.set_keyring(PlaintextKeyring())
    config_name = os.getenv('FLASK_ENV', 'cryptodataaccess.config.DevelopmentConfig')
    cfg = import_string(config_name)()
    cfg.LOCAL_MONGO_USERNAME = get_password('cryptodataaccess', 'LOCAL_MONGO_USERNAME')
    cfg.LOCAL_MONGO_PASSWORD = get_password('cryptodataaccess', cfg.LOCAL_MONGO_USERNAME)
    cfg.CENTRAL_MONGO_USERNAME = get_password('cryptodataaccess', 'CENTRAL_MONGO_USERNAME')
    cfg.CENTRAL_MONGO_PASSWORD = get_password('cryptodataaccess', cfg.CENTRAL_MONGO_USERNAME)
    return cfg

